import os, sys, zipfile, time, xml.dom.minidom, traceback, urllib, urllib2, shutil, locale, subprocess
from os.path import *
from datetime import datetime

# ***** include the biocase.lib directory in the python sys path for importing *****
sys.path.insert(0, join(dirname(__file__), os.pardir, os.pardir))
from biocase.archive.general import *

xmla = sys.argv[1]
dwca = sys.argv[2]
dsa = sys.argv[3]
archiveFilename =  unicode(sys.argv[4])
archivePath = unicode(sys.argv[5])
downloadPath = unicode(sys.argv[6])
javaBin = sys.argv[7]
archiveLib = sys.argv[8]
kettleLib = sys.argv[9]
kettleHome = sys.argv[10]
javaMem = sys.argv[11]
sortSize = sys.argv[12]
transformer = sys.argv[13]

def log(logFile, message, tag = 'INFO'):
    l = open(logFile, 'a')
    ts = datetime.now()
    for line in message.splitlines():
        l.write('%-5s %02i-%02i %02i:%02i:%02i,%03i - %s\n' %(tag, ts.day, ts.month, ts.hour, ts.minute, ts. second, ts.microsecond/1000, line))
    l.close()

def create_xml():
    logFile = join(archivePath, archiveFilename + ".log")
    queryFile = join(archivePath, archiveFilename + ".query.xml")
    configFile = join(archivePath, archiveFilename + ".config.xml")

    # Load configuration from file
    dom = xml.dom.minidom.parse(configFile)
    dsa = str(dom.getElementsByTagName("dsa")[0].firstChild.data)
    url = str(dom.getElementsByTagName("wrapperurl")[0].firstChild.data)
    ticket = str(dom.getElementsByTagName("ticket")[0].firstChild.data)
    download_folder = join(str(dom.getElementsByTagName("downloadfolder")[0].firstChild.data), dsa)
    maxErrors = int(dom.getElementsByTagName("consecutiveerrors")[0].firstChild.data)

    dom = xml.dom.minidom.parse(queryFile)
    n = dom.getElementsByTagName("responseFormat")[0]
    limit = int(n.getAttribute("limit"))

    log(logFile, "Archive file:    %s \nQuery file:      %s \nConfig file:     %s \nDatasource:      %s \nDownload folder: %s \nWrapper URL:     %s \nTicket:          %s \nLimit:           %i"
        % (join(archivePath, archiveFilename), queryFile, configFile, dsa, download_folder, url, ticket, limit), 'INFO')

    procFilename = join(archivePath, archiveFilename + ".proc")
    proc = open(procFilename, "w")
    proc.write("Pid: "+str(os.getpid()))
    proc.write("Process started " + datetime.now().ctime())
    proc.close()

    firstrun = 1
    consecutiveErrors = 0;
    archiveAppend = "0"
    endMessage = ""
    no_records_found = False

    while True:
        try:
            # check if cancel
            proc = open(procFilename)
            cancel = proc.read()
            proc.close()
            if cancel == "cancel":
                log(logFile, "CANCEL received.")
                break;
            
            start = int(n.getAttribute("start"))		
            queryData = dom.toxml('utf-8')
            log(logFile, "Requesting records %s - %s" % (start+1, start + limit))

            data = urllib.urlencode([('query', queryData)])
            req = urllib2.Request(url + "&archiverequest=1&archiveappend="+archiveAppend+"&archivefilename="+archiveFilename+"&ticket="+ticket)
            fd = urllib2.urlopen(req, data)
            data = fd.read()
            fd.close()

            r = xml.dom.minidom.parseString(data.encode("utf-8"))
            r.documentElement.tagName == "response"

            hits = 0
            rcount = 0
            dropped = 0
            status = "N/A"
            message = "N/A"
            try:
                hits = int(r.getElementsByTagName("hits")[0].firstChild.data)
                rcount = int(r.getElementsByTagName("rcount")[0].firstChild.data)
                dropped = int(r.getElementsByTagName("dropped")[0].firstChild.data)
                status = str(r.getElementsByTagName("status")[0].firstChild.data)
                message = str(r.getElementsByTagName("message")[0].firstChild.data)
            except:
                pass

            fetched = hits - start
            if firstrun == 1 and hits == 0:
                no_records_found = True
            
            log(logFile, "  Hits:    %i\n  Count:   %i\n  Dropped: %i\n  Status:  %s\n  Message: %s"
                % (hits, rcount, dropped, status, message))

            if status != "ok":
                consecutiveErrors += 1
                if endMessage == "":
                    endMessage = "--------------------- Error Summary ------------------------\n"
                    endMessage += "One or several errors occurred during the archiving process:\n"
                endMessage += "An error occurred while requesting records "+str(start+1)+" through "+str(start + limit)+":\nStatus: "+status+"\nMessage: "+message+"\n"
                log(logFile, "An error occurred during this request", 'ERROR')
                if consecutiveErrors >= maxErrors:
                    log(logFile, str(maxErrors)+" consecutive errors, giving up further tries. Exit.", 'ERROR')
                    break;
            else:
                consecutiveErrors = 0

            if rcount + dropped == 0 and status == "ok":
                #log("Archiving finished")
                break;

            if rcount + dropped == hits and status == "ok":
                #log("Archiving finished")
                break;

            if rcount + dropped  < limit:
                if firstrun == 1 and status == "ok":				
                    log(logFile, "Adjusting fetch size limit to " + str(rcount))
                    limit = rcount;
                elif status == "ok":
                    #log("Archiving finished")
                    break;
                        
            nextStart = start + limit
            n.removeAttribute("start")
            n.setAttribute("start", str(nextStart))
            archiveAppend = "1"
            firstrun = 0
                    
        except:
            n.removeAttribute("start")
            n.setAttribute("start", str(start + limit))
            archiveAppend = "1"

            et, ev, tb = sys.exc_info()
            error = str(sys.exc_info()[1])
            while tb :
                co = tb.tb_frame.f_code
                filename = "Filename = " + str(co.co_filename)
                line_no =  "Error Line # = " + str(traceback.tb_lineno(tb))
                error += "\n" + filename
                error += "\n" + line_no
                tb = tb.tb_next
            error += "\n"

            if endMessage == "":
                endMessage = "One or several errors occurred during the archiving process:\n"+error
            endMessage += "An error occurred while requesting records "+str(start+1)+" through "+str(start + limit)+":\n"+error
            log(logFile, "An error occurred during this request: "+error, 'ERROR')
            consecutiveErrors += 1
            if consecutiveErrors >= maxErrors:
                log(logFile, str(maxErrors)+" consecutive errors, giving up further tries. Exit.", 'ERROR')
                break;

    if endMessage <> '':
        log(logFile, endMessage + '-'*60, 'ERROR')

    # Sleep for a second to make sure that the log is displayed even in the case we came here very fast
    time.sleep(1)

    ret_code = 1
    if consecutiveErrors >= maxErrors or cancel == 'cancel':
        log(logFile, 'ERROR: Archiving failed!', 'ERROR')
    else:
        # No records found? Add a message to the log, do not copy anything
        if no_records_found:
            log(logFile, 'No records to be archived. Make sure that this mapping returns any results!', 'ERROR')
            log(logFile, 'ERROR: Archiving failed!', 'ERROR')
        else:
            # Otherwhise, if no errors occured: Copy files to download folder
            try:
                if not exists(download_folder):
                    os.makedirs(download_folder)
                shutil.copy(join(archivePath, archiveFilename + '.zip'), download_folder)
                shutil.copy(configFile, download_folder)
                os.remove(join(archivePath, archiveFilename + '.zip'))
                os.remove(configFile)
                log(logFile, 'SUCCESS: Archiving finished.')
                shutil.copy(logFile, download_folder)
                ret_code = 0
            except Exception, err:
                log(logFile, 'Archive could not be moved to the download folder:', 'ERROR')
                log(logFile, str(err), 'ERROR')
                log(logFile, 'ERROR: Archiving failed!', 'ERROR')

    # Last, return ret_code
    os.remove(procFilename)
    return ret_code

def create_dwca():
    start = time.clock()
    logFile = join(archivePath, DWCA_SCHEMA_NAME + '.log')
    dupFile = join (archivePath, 'duplicate.txt')
    proc_dwca = join(archivePath, DWCA_SCHEMA_NAME + '.proc')
    tmpPath = join (archivePath, 'tmp')
    archiveDldFile = join(downloadPath, archiveFilename + '.zip')
    datasetList = join (tmpPath, 'datasets.list')
    dwca_files = ['occurrence.txt', 'image.txt', 'identification.txt', 'meta.xml', 'eml.xml']

    # First, create proc file
    try:
        proc = open(proc_dwca, 'w')
        failed = False

        try:
            # Then, try to remove old dup file
            try:
                if exists(dupFile):
                    os.remove(dupFile)
            except Exception, err:
                log(logFile, 'Error trying to remove old status file: ' + str(err), 'ERROR')
                dupFile = None
                raise

            # Start java process
            log(logFile, 'Starting the transformation engine\nJava heap space used is %sM, sort size is %s' % (javaMem, sortSize))
            args = ['%s'                    % (javaBin),
                    '-Djava.ext.dirs=%s'    % (kettleLib),
                    '-Djava.io.tmpdir=%s'   % (tmpPath),
                    '-DKETTLE_HOME=%s'      % (kettleHome),
                    '-Xmx%sm'               % (javaMem),
                    'org.pentaho.di.kitchen.Kitchen',
                    '-file=%s'              % (join(archiveLib, 'dwca.kjb')),
                    '-param:sort_size=%s'   % (sortSize),
                    '-param:base_dir=%s'    % (archivePath),
                    '-param:archive=%s'     % (archiveDldFile),
                    '-param:transformer=%s' % (transformer)]
            size = getsize(logFile)
            l = open(logFile, 'a')
            l.seek(size)
            process = subprocess.Popen(args, shell=False, stdout=l, stderr=l)
            l.close

            # Then, write pid to file
            proc.write(str(process.pid))
            proc.close()
            
            # Waiiiiiiiiiiiiiiiiit
            ret_code = process.wait()
            log(logFile, 'Transformation ended with return code ' + str(ret_code))
            
            if ret_code == 0:
                log(logFile, 'Finishing DarwinCore Archives:')
                try:
                    # Read list of datsets
                    log(logFile, 'Reading dataset list')
                    datasets = []
                    f = open(datasetList, 'r')
                    for line in f:
                        datasets.append(unicode(line.rstrip(), 'utf-8'))
                    f.close()
                    
                    # Create zip files
                    log(logFile, 'Creating zip files')
                    for dataset in datasets:
                        zip = zipfile.ZipFile(join(tmpPath, dataset + '.dwca.zip'), 'w', zipfile.ZIP_DEFLATED, True)
                        for fname in dwca_files:
                            src_fname = join(tmpPath, dataset + ' ' + fname)
                            if exists(src_fname):
                                zip.write(src_fname, fname)
                        zip.close()

                    # Remove all old DwC Archives
                    log(logFile, 'Removing old files from download folder')
                    l = os.listdir(downloadPath)
                    for fname in l:
                        if fname.endswith("dwca.zip") or fname.endswith(DWCA_SCHEMA_NAME + ".log"):
                            os.remove(join(downloadPath, fname))

                    # Move zipfiles
                    log(logFile, 'Moving archives and log file to download folder')
                    for dataset in datasets:
                        shutil.move(join(tmpPath, dataset + '.dwca.zip'), downloadPath)

                    # Success message, copy log
                    log(logFile, 'SUCCESS: The DwC Archive creation completed successfully!')
                    shutil.copy(logFile, downloadPath)

                # Problem during finishing the archive
                except Exception, err:
                    log(logFile, str(err), 'ERROR')
                    failed = True

            # Pentaho transformation failed
            else:
                failed = True

        # Java process failed (or removing the dup file didn't work)
        except Exception, err:
            if dupFile is not None:
                log(logFile, 'Java process could not be started: ' + str(err), 'ERROR')
                log(logFile, 'Check the Test libs page to make sure Java is properly installed and configured.', 'ERROR')
            failed = True

        finally:
            # If we came here in less than five seconds, we need to wait to make sure that the log is properly displayed in the UI
            elapsed = time.clock() - start
            if elapsed < 5:
                time.sleep(5-elapsed)

            # Add some error messages
            if failed:
                log(logFile, 'ERROR: The DwC Archive creation failed!', 'ERROR')

                # Check for the existence of a duplicate.txt
                if dupFile is not None and exists(dupFile):
                    try:
                        dup = open(dupFile, 'r')
                        for line in dup:
                            cat_nr = line.rstrip()
                        dup.close()
                        log(logFile, 'ERROR: Duplicate catalog number "%s" found in the dataset. Please eliminate it, recreate the XML archive and retry.' % (cat_nr), 'ERROR')
                    except Exception, err:
                        log(logFile, 'Duplicate catalog number file could not be opened: ' + str(err), 'WARN')

            # Remove tmp folder
            try:
                if exists(tmpPath):
                    shutil.rmtree(unicode(tmpPath))
            except Exception, err:
                log(logFile, 'Temp folder could not be removed: ' + str(err), 'WARN')

            # Last, close log and remove proc file
            if not proc.closed:
                proc.close()
            os.remove(proc_dwca)            
            
    # Already the proc file failed: No clean-up required
    except Exception, err:
        log(logFile, "Proc file could not be created: " + str(err), 'ERROR')
        time.sleep(5)

# Main
if xmla == 'True':
    ret = create_xml()
    # If both types should be created, only start trnasformation if XML archiving was successful
    if dwca == 'True':
        if ret == 0:
            create_dwca()
else:
    create_dwca()
