# -*- coding: utf-8 -*-
'''
$RCSfile: $
$Revision: 1234 $
$Author: j.holetschek $
$Date: 2012-11-26 10:15:30 +0100 (Mo, 26. Nov 2012) $
'''

from datetime import datetime
import os

# execfile('c:/Workspace/bps2/lib/biocase/adjustpath.py')

# Some constants for archiving
NS_ABCD206 = 'http://www.tdwg.org/schemas/abcd/2.06'
NS_HISPID5 = 'http://www.chah.org.au/schemas/hispid/5'

transformers = {NS_ABCD206: 'abcd2.ktr',
                NS_HISPID5: 'hispid5.ktr'}

DWCA_SCHEMA_NAME = 'DwCA'

# Get configuration
import biocase.configuration
cfg = biocase.configuration.Cfg()

def is_idle(dsa):
    '''Returns the archiving engine status for the given data source'''
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')
    else:
        idle = True
        archivePath = os.path.join(cfg.archiveWorkLocator, dsa)
        for fname in os.listdir(archivePath):
            if fname.endswith(".proc"):
                idle = False
                break
        return idle

def filenamizeDatasetTitle(title):
    ' Turns a dataset title into a string that can be used as a filename (by replacing dissallowed chars)'
    if title is None:
        return None
    else:
        return title.replace('/', '_').replace('\\', '_').replace(':', '_').replace('?', '_').replace('*', '_').replace('"', "'").replace('<', '[').replace('>', ']').replace(' ', '_').replace('|', '_')

def list_archives(dsa):
    ' Returns a list of archive files for the given data source as a dictionary'
    ' Key: archive name (without suffixes)'
    ' Value: Tuple (archive URL, modified date)'
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')
    else:
        ret = {}
        dldPath = os.path.join(cfg.archiveDownloadLocator, dsa)
        baseURL = cfg.getWebappURL() + cfg.archiveDownloadLocator[len(cfg.wwwLocator) + 1:].replace('\\', '/') + '/' + dsa + '/'
        if os.path.exists(dldPath):
            l = os.listdir(dldPath)
            for fname in l:
                if fname.endswith(".zip"):
                    # Split filename
                    fname_head = fname[:-4]
                    if fname_head.endswith(".dwca"):
                        fname_head = fname_head[:-5]
                    # Insert into dict
                    modified = datetime.fromtimestamp(os.stat(os.path.join(dldPath, fname)).st_mtime)
                    ret[fname_head] = (baseURL + fname, modified)

        return ret