# -*- coding: utf8 -*-
'''
DB MODULE - ODBC for TSQL
*******************************
***  DB MODULE - ODBC for TSQL
*******************************
$RCSfile: dbmod_odbc_tsql.py,v $
$Revision: 1231 $
$Author: j.holetschek $
$Date: 2012-11-21 15:12:34 +0100 (Mi, 21. Nov 2012) $

A native biocase.wrapper database module to access MS SQL Server databases.
Basedon pymssql 
http://pymssql.sourceforge.net/
'''


import os
from string                               import join
from datetime                             import datetime
from biocase.wrapper.errorclasses         import *
from biocase.wrapper.typeconverter        import DBtransformerClass
from biocase.wrapper.sql.base             import sqlBase
from biocase.wrapper.dbmod.base_dbmod     import DBBasemodule

import logging
log = logging.getLogger("pywrapper.dbmod")

#############################################
#
# CUSTOMIZATION for SQL Server
#
# OVERLOAD BASE CLASSES AND OTHER METHODS
#
#############################################

# uses inherited ANSI SQL apart from the following.
sqlBase.templates['attribute'] = '[#1].[#2]'
sqlBase.templates['table'] = '[#1]'
sqlBase.templates['select-limit'] = 'SELECT DISTINCT TOP #3 #1 FROM #2'



# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
class DBmodule(DBBasemodule):
	__version__ = '0.1 using pymssql'
	__dbmod__    = u'pymssql Server module'
	
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __init__(self, psfObj):
		# call createDBconnection method
		DBBasemodule.__init__(self, psfObj)
		# dont use parameters but integrate all parameters into SQL strings:
		self.trans.integrateParametersInSQLString()

	# ------------------   FUNCTION OVERLOAD OF BASE DBMOD MODULE   ------------------
	# --------------------------------------------------------------------------------
	def createDBconnection(self):
		try:
			import pymssql
		except:
			raise DbModuleMissingError('pymssql')
		self.__class__.__version__ = self.__class__.__version__[:-1]+pymssql.__version__
		# connect
		try:
			return pymssql.connect(host=self.psfObj.dbIP,user=self.psfObj.user,password=self.psfObj.password,database=self.psfObj.database)
		except:
			raise ConnectionError()

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def getTables(self):
		tables = []
		self.c = self.conn.cursor()
		self.c.execute("SELECT name FROM sysobjects where type in ('U', 'V') and status > 0")
		for t in self.c.fetchall():
			tables.append(str(t[0]))
		self.c.close()
		return tables

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def getColumns(self, _table):
		columns = []
		self.c = self.conn.cursor()
		try:
			self.c.execute("SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS where table_name = '%s'" % (_table))
			for c in self.c.fetchall():
				columns.append(str(c[0]))
		except:
			log.error("Couldn't retrieve column info for table " +  _table)
		finally:
			self.c.close()
		return columns

	def hasMetaInfo(self):
		return True