#!C:\Programme\Python25\python.exe
# -*- coding: UTF-8 -*-
import os, sys, subprocess, signal, datetime, hashlib
from os.path import *

# ***** include the biocase.lib directory in the python sys path for importing *****
execfile( os.path.abspath( os.path.join( os.path.dirname( __file__ ), os.pardir, 'lib', 'biocase', 'adjustpath.py' ) ))

from biocase.configtool.general import *
from biocase.datasources import Datasource
from biocase.archive.general import *

print "Content-Type: text/plain\n"

# First, check the action values
action = form.getvalue('action')
if action is None:
    action = 'log'
else:
    action = action.lower()
if action not in ['log', 'xml', 'dwc', 'dwc_only', 'cancel']:
    print "Unrecognized value for parameter action. Only 'log', 'xml', 'dwc' and 'cancel' are allowed."
    sys.exit()

# dsa is the only mandatory parameter; all others are optional
dsa = form.getvalue('dsa')
cfg = biocase.configuration.Cfg()
try:
    dsaObj = Datasource(name=dsa)
    psfObj = dsaObj.getPSFObj()
except:
    print "Invalid Datasource given!"
    sys.exit()

# Check authentication (ticket indicates we've been invoked from the UI)
# Ticket is the MD5-encrypted psf/biocase password
vticket = hashlib.md5(psfObj.adminpassword if psfObj.adminpassword else cfg.server.adminpassword).hexdigest()
if form.getvalue('ticket'):
    from_ui = True
    ticket = form.getvalue('ticket')
    if ticket != vticket:
        print "Authentication failed!"
        sys.exit()
else:
    from_ui = False
    pw = form.getvalue('pw')
    if pw is None or pw == "":
        print "No password provided!"
        sys.exit()
    else:
        if pw == (psfObj.adminpassword if psfObj.adminpassword else cfg.server.adminpassword):
            ticket = vticket
        else:
            print "Authentication failed!"
            sys.exit()

# set defaults for optional parameterss
filesize = form.getvalue('filesize')
errors = form.getvalue('errors')
if filesize is None or not filesize.isdigit():
    filesize = "1000"
if errors is None or not errors.isdigit():
    errors = "1"
            
# Schema: If it's empty when called directly, find the most likely schema
schema = form.getvalue('schema')
if schema is None:
    try:
        sl = dsaObj.getSchemaListObj()
        # try to find the ABCD2 CMF
        # as an alternative, sl.getSchemaByNS(NS_ABCD206) could be used here
        cmf = [e for e in sl if e.NS == NS_ABCD206]
        # if not, try to find the HISPID5 CMF
        if cmf == []:
            cmf = [e for e in sl if e.NS == NS_HISPID5]
        # if not, use the first one we get
        if cmf == []:
            cmf = sl[:1]
        # get name of schema mapping (remove trailing .xml) and set filename
        schema = cmf[0].name[:-4]
    except:
        print "The specified datasource doesn't have any schemas mapped!"
        sys.exit()    

# Get schema namespace
try:
    cmf = dsaObj.getSchemaListObj().getSchemaByName(schema + '.xml')
    schemaNs = cmf.NS
except:
    print "The specified schema mapping doesn't exist for this datasource!"
    sys.exit()

# For DwC, check namespace and set transformer
transformer = None
if action in ('dwc', 'dwc_only'):
    if schemaNs in transformers.keys():
        transformer = transformers[schemaNs]
    else:
        print "DarwinCore archives can only be created for the ABCD 2.06 and HISPID 5 schemas, so make sure the datasource supports one of these. If you don't specify a schema in the request, BioCASe will try to use ABCD 2.06, otherwise HIDPID 5."
        sys.exit()

# Set some variables
log = schema + '.log'
wrapperurl = dsaObj.getBioCASeAccessPoint()
archivePath = join(cfg.archiveWorkLocator, dsa)
archiveBin = join(cfg.archiveLibLocator, "archive.py")
proc_dwca = join(archivePath, DWCA_SCHEMA_NAME + '.proc')
proc_xml = join(archivePath, schema + ".proc")
log_dwca = join(archivePath, DWCA_SCHEMA_NAME + '.log')
log_xml = join(archivePath, schema + '.log')
dldPath = join(cfg.archiveDownloadLocator, dsa)

def log():
    print 'Idle.' if is_idle(dsa) else 'Processsing.'
    print "\nBelow you'll find the log of the latest archiving process.\n" + '*' * 120
    # Append the latest log
    l = [join(archivePath, s) for s in os.listdir(archivePath) if s.endswith('.log')]
    l.sort(key = lambda s: getmtime(s))
    if len(l) == 0:
        print "-- No archiving process for this datasource so far --"
    else:
        for line in open(l[-1]).readlines():
            print line,
        # If the latest process was DwCA, also attach the latest XML log
        if l[-1].endswith(DWCA_SCHEMA_NAME + '.log') and len(l) > 1:
            print "\nBelow you'll find the log of the corresponding XML archiving process.\n" + '*' * 120
            for line in open(l[-2]).readlines():
                print line,

def create(xml, dwca):
    try:
        if xml:
            # Write config/query file
            q = open(join(archivePath, schema + ".query.xml"), "w")
            q.write("<?xml version='1.0' encoding='UTF-8'?>\n")
            q.write("<request xmlns='http://www.biocase.org/schemas/protocol/1.3'>\n")
            q.write("  <header><type>search</type></header>\n")
            q.write("  <search>\n")
            q.write("    <requestFormat>"+schemaNs+"</requestFormat>\n")
            q.write("    <responseFormat start='0' limit='"+filesize+"'>"+schemaNs+"</responseFormat>\n")
            q.write("    <count>false</count>\n")
            q.write("  </search>\n")
            q.write("</request>\n")
            q.close()

            c = open(join(archivePath, schema + ".config.xml"), "w")
            c.write("<?xml version='1.0' encoding='UTF-8'?>\n")
            c.write("<config>\n")
            c.write("<dsa>" + dsa + "</dsa>\n")
            c.write("<archivefilename>" + schema + ".zip</archivefilename>\n")
            c.write("<downloadfolder>" + cfg.archiveDownloadLocator + "</downloadfolder>\n")
            c.write("<wrapperurl>" + wrapperurl + "</wrapperurl>\n")
            c.write("<schema>"+schema+"</schema>\n")
            c.write("<schemans>"+schemaNs+"</schemans>\n")
            c.write("<consecutiveerrors>"+errors+"</consecutiveerrors>\n")
            c.write("<fromui>"+str(from_ui)+"</fromui>\n")            
            c.write("<ticket>"+ticket+"</ticket>\n")
            c.write("</config>\n")
            c.close()

            # Try to open (an empty) logfile
            l = open(log_xml, 'w')
            l.close()

        if dwca:
            # Try to open (an empty) logfile
            l = open(log_dwca, 'w')
            l.close()

        # We need to set stdin and stdout because of a bug in Python:
        # Leaving these as None will result in a "handle not valid" error on IIS/Windows
        proc = subprocess.Popen([sys.executable, archiveBin, str(xml), str(dwca), dsa, schema, archivePath, dldPath,
                          cfg.server.java, cfg.archiveLibLocator, cfg.kettleLibLocator, cfg.archiveWorkLocator, cfg.server.java_mem, cfg.server.sort_size,
                          str(transformer)], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        for err in proc.stderr:
            print err,
        sys.exit()

    except Exception, err:
        print err

def cancel():

    def cancel_xml(proc_file):	
        p = open(proc_file, "w")
        p.write("cancel")
        p.close()
        if from_ui:
            print "Cancellation requested. Waiting for the running statement to finish..."

    def dwca_cancel(proc_file):
        # Read process id
        q = open(proc_file, 'r')
        for line in q:
            id = line
        q.close()

        # First, try the Linux version
        try:
            try:
                os.kill(int(id), signal.SIGTERM)
                if from_ui:
                    print "Process terminated by sending TERMINATE event."
            except AttributeError:
                # Wrong OS; raising error to get to get to Windows section
                raise
            except:
               print "Sorry, couldn't terminate process. Please wait for it to finish or cancel it manually using ps -kill (look for java)."

        # If that fails, try Windows taskkill with option f (forcefull)
        except:
            res = subprocess.call("taskkill /f /pid %s" % id)
            if res == 0:
                if from_ui:
                    print "Process terminated using Windows Taskkill."
            else:
                print "Sorry, couldn't terminate process. Please wait for it to finish or cancel it manually using the task manager (look for java.exe)."

    # Try to find the proc file
    for fname in os.listdir(archivePath):
        if fname.endswith('.proc'):
            proc_file = join(archivePath, fname)
            if fname.endswith(DWCA_SCHEMA_NAME + '.proc'):
                dwca_cancel(proc_file)
            else:
                cancel_xml(proc_file)

            # Before returning, wait until proc file disappears (only if not from UI)
            if not from_ui:
                while exists(proc_file):
                    time.sleep(5)
                print "Process cancelled."
            return
        
    # No proc file found
    print "No process running."

# Main
if action == 'log':
    log()
elif action == 'cancel':
    cancel()
else:
    # Check if a process is already running
    if is_idle(dsa):
        if action == "xml":
            create(True, False)
        elif action == "dwc_only":
            create(False, True)
        elif action == 'dwc':
            create(True, True)
    else:
        print "An archiving process is already running. Please wait for it to finish."
