#!C:\Programme\Python25\python.exe
# -*- coding: UTF-8 -*-

'''
$RCSfile: datasource.py,v $
$Revision: 1217 $
$Author: markus $
$Date: 2012-11-09 12:30:58 +0100 (Fr, 09. Nov 2012) $
'''
import os.path

import os, sys
# ***** include the biocase.lib directory in the python sys path for importing *****
execfile( os.path.abspath( os.path.join( os.path.dirname( __file__ ), os.pardir, os.pardir, 'lib', 'biocase', 'adjustpath.py' ) ))
execfile( os.path.abspath( os.path.join( os.path.dirname( __file__ ), os.pardir, os.pardir, 'lib', 'biocase', 'appinit.py' ) ))


from biocase.configtool.general import *
from biocase.configtool.cmfupgrade import cmfUpgrade
from biocase.cmfinfo            import CMFTemplateListClass
from biocase import appinit, __version__
import xml.dom.minidom
from xml.dom.minidom import Node
import xml.sax
import pprint
import urllib2, urllib
import time, datetime


############################################################################################################
#
#   MAIN
#
#===========================================================================================================

# check authentication!
if dsaObj is None:
    execfile( os.path.abspath( os.path.join( os.path.dirname( __file__ ), 'main.cgi' ) ))
    sys.exit()

# check authentication!
authorize(form, dsa=dsa)


# use the datasource.html template to get datasource overview
tmpl = PageMacro('Content', PageMacro.DELMODE)
tmpl.load('Content', os.path.join(templateDir, '_datasource.html'))

# PSF
psfObj = dsaObj.getPSFObj(tmp=False)

# upgrade object
upgradeObj = cmfUpgrade()

# Indicate whether a mapping has been removed or recyled
# Required to trigger PSF update later
recyclingOperation = False

def del_schema(schema, dsaObj):
    old = os.path.join(dsaObj.getDsaDir(), schema)
    new = dsaObj.getBakFileName(schema)
    log.debug( "Removing CMF %s from datasource" %old)
    if os.path.isfile(new):
        os.unlink(new)
    os.rename(old, new)
   
#
# ACTIONS
#
if action == 'del':
    # "remove" CMF. renames it to a backup file name (= recycle bin), from which it can be restored later
    schema = form.getfirst('schema', None)
    del_schema(schema, dsaObj)
    # refresh dsaObj
    dsaObj = Datasource(name=dsa)
    recyclingOperation = True    

elif action == 'transfer':
    # upgrade an existing cmf
    oldCmf = os.path.join( dsaObj.getDsaDir(), form.getfirst('schema',None) )
    newSchema = form.getfirst('targetschema') + '.xml'
    newCmfObj, missingConcepts, upgradeTuple = upgradeObj.getUpgradedCmfObj(oldCmf, newSchema)
    if newCmfObj is None:
        log.error( "ERROR ! CMF could not be upgraded. Unknown error" )
    else:
        newCmf = os.path.join( dsaObj.getDsaDir(), upgradeTuple[0] )
        # Check whether there is already a cmf for that schema
        existingCmf = dsaObj.getSchemaListObj().getSchemaByNS(newCmfObj.rootCMFElementObj.Namespace)
        if existingCmf is not None:
            tmpl['updateHint'] = "alert('INFO: A mapping file with the same namespace as the selected schema was already existent (" + existingCmf.name + "). It has been put into the recyling bin if you change your mind.');"
            del_schema('cmf_' + existingCmf.name, dsaObj)
        newCmfObj.writeCMF(newCmf)
        # refresh dsaObj
        dsaObj = Datasource(name=dsa)
 
elif action == 'restore':    
    schemaFile = form.getfirst('schemafile',None)
    schema = schemaFile[5:-8]
    old = os.path.join(dsaObj.getDsaDir(), schemaFile)
    new = os.path.join(dsaObj.getDsaDir(), "cmf_" + schema)    
    if os.path.isfile(new):
        tmpl['restoreError'] = "alert('A " + schema + " schema is already active. Please remove it first if you want to restore the " + schemaFile + " file');"
    else:
        os.rename(old, new)
    
    # refresh dsaObj
    dsaObj = Datasource(name=dsa)
    recyclingOperation = True

elif action == 'purge':
    # "purge" CMF.    
    schemaFile = os.path.join(dsaObj.getDsaDir(), form.getfirst('schemafile',None))
    os.unlink(schemaFile)
    
#
# FILL TEMPLATE
#
# general
tmpl['dsa'] = dsa
tmpl['wrapper_url'] = dsaObj.getBioCASeAccessPoint()
tmpl['ServiceTitle'] = 'BioCASe Provider Software %s' % __version__

# dbconnection status
tmpl['status'] = dsaObj.getDBConnectionStatus()

# number of aliases in PSF
tmpl['num_ta'] = str(len( psfObj.getTableAliasList() ))

# list of dsa schemas
slObj = dsaObj.getSchemaListObj()
log.debug("DSA CMFList: %s"%str(slObj))

# list of duplicate dsa schemas with the same NS
origSchemas = {}
duplicateList = []
for sObj in slObj:
    if origSchemas.has_key(sObj.NS):
        duplicateList.append( {'schema':sObj.name, 'schemaorig':origSchemas[sObj.NS].name} )
    else:
        origSchemas[sObj.NS] = sObj
tmpl.expand('Content', 'schemaduplicatelist', duplicateList)

#archive
dldPath = os.path.join(cfg.archiveDownloadLocator, dsa)
archivePath = os.path.join(cfg.archiveWorkLocator, dsa)
if not os.path.exists(archivePath):
    os.makedirs(archivePath)
if not os.path.exists(dldPath):
    os.makedirs(dldPath)
	
archives = []
dwcas = []
l = os.listdir(unicode(dldPath))
for f in l:
    if f.endswith(".config.xml"):
        dom = xml.dom.minidom.parse(os.path.join(dldPath, f))
        s = str(dom.getElementsByTagName("schema")[0].firstChild.data)		
        a = {}
        a["schema"] = s		
        a["archive"] = f[:-11] + '.zip'
        a["archivefile"] = os.path.join(dldPath, f[:-11] + '.zip')
        a["procfile"] = archivePath + os.sep + f[:-11] + ".proc"
        if os.path.exists(a["procfile"]):
            a["created"] = "Processing..."
            a["processing"] = True
        else:
            if os.path.exists(a["archivefile"]):
                a["created"] = time.strftime("%Y-%m-%d %H:%M:%S",time.localtime(os.path.getmtime(a["archivefile"])))
            else:
                a["created"] = "File missing!"
            a["processing"] = False
        archives.append(a)
    if f.endswith('.dwca.zip'):
        a = {}
        a["archive"] = f
        a["archivefile"] = os.path.join(dldPath, f)
        if os.path.exists(os.path.join(archivePath, "DwCA.proc")):
            a["created"] = "Processing..."
        else:
            a["created"] = time.strftime("%Y-%m-%d %H:%M:%S",time.localtime(os.path.getmtime(a["archivefile"])))
        dwcas.append(a)

# create existing cmf schema list...
DsaSchemaList = []
for sObj in slObj:
    entry = {}
    entry['schema'] = sObj.name
    entry['schema_file'] = os.path.split(sObj.file)[1]
    entry['namespace'] = sObj.NS
    entry['num_mappings'] = str(sObj.mappings)
    entry['num_concepts'] = str(sObj.concepts)
    entry['schemalist.upgrades'] = [ {'ns': t[0], 'targetschema': t[1][:-4]} for t in upgradeObj.listPossibleUpgradeTuples(sObj.NS) if t[1][4:] != sObj.name]
    if len(entry['schemalist.upgrades']) == 0:
        entry['supress_upgrade_start'] = '<!--'
        entry['supress_upgrade_end'] = '-->'
    else:
        entry['supress_upgrade_start'] = ''
        entry['supress_upgrade_end'] = ''
    
    entry['schemalist.archivelist'] = []
    for a in archives:	
        archive = {}
        if a['schema']+".xml" == sObj.name:
            archive['archive'] = a["archive"]
            archive['created'] = a['created']
            archive['link'] = '<a href="archive.cgi?dsa='+dsa+'&archive='+a["archive"]+'">Details</a>'
            entry['schemalist.archivelist'].append(archive)
            
			
    if len(entry['schemalist.archivelist']) == 0:
        entry['a_hide_s'] = "<!--"
        entry['a_hide_e'] = "-->"
    else:
        entry['a_hide_s'] = ""
        entry['a_hide_e'] = ""

    DsaSchemaList.append( entry )
    

tmpl.expand('Content', 'schemalist', DsaSchemaList)


# UPDATE PSF WITH THESE SCHEMAS ?
CmfUpdates = [sObj.getLastUpdated() for sObj in slObj if sObj is not None]
if recyclingOperation or (len(CmfUpdates) > 0):
    if len(CmfUpdates) > 0:
        lastCMFupdate = max(CmfUpdates)
    else:
        lastCMFupdate = None
    psfUpdate = psfObj.getPSFUpdatedWhen()
    if recyclingOperation or (lastCMFupdate > psfUpdate):
        log.debug("Need to update PSF with CMF list. Last PSF update at %s, last CMF update %s" %( str(psfUpdate), str(lastCMFupdate)))
        psfObj.delAllSchemas()
        for sObj in slObj:
            psfObj.addSchema(sObj.NS, sObj.file)
        psfObj.writePSFile()
    else:
        log.debug("No need to update PSF with CMF list. Last PSF update at %s, last CMF update %s" %( str(psfUpdate), str(lastCMFupdate)))
else:
    log.debug("No CMFs found. PSF will not be updated")
    
# list of cmf templates
tmplSlObj = slObj
tmplSlObj = CMFTemplateListClass()
log.debug("Full Template CMFList: %s"%str(tmplSlObj))
# dont show already configured schemas. Identify duplicates by their NS
for sObj in slObj:
    tmplSlObj.removeSchema(NS=sObj.NS)
dropDown = getDropDownOptionHtml( dict([('--select--','--select--')] + [(sObj.name,os.path.split(sObj.file)[1])for sObj in tmplSlObj]), '--select--')
tmpl['newschema_options'] = dropDown


# recycle bin
deletedSchemas = dsaObj.getDeletedSchemaListObj()
DsaDeletedSchemaList = []
for ds in deletedSchemas:
    s = {}
    s['deletedschema'] = ds.name    
    s['schema_file'] = os.path.split(ds.file)[1]    
    s['modtime'] = time.strftime("%Y-%m-%d %H:%M:%S",time.localtime(os.path.getmtime(ds.file)))
    s['num_mappings'] = str(ds.mappings)
    s['num_concepts'] = str(ds.concepts)
    # if schema with the same NS is already existing, don't allow restoring
    if origSchemas.has_key(ds.NS):
        s['restore'] = ''
    else:
        s['restore'] = 'Restore'
    DsaDeletedSchemaList.append(s)

tmpl.expand('Content', 'recycle', DsaDeletedSchemaList) 
if len(DsaDeletedSchemaList) == 0:    
    tmpl['recycleBinEmptyMessage'] = "Recycle bin is empty."
    tmpl['hideRecycleBinStart'] = "<!--"
    tmpl['hideRecycleBinEnd'] = "-->"

# Last, DwCA

tmpl.expand('Content', 'dwcalist', dwcas)
if len(dwcas) == 0:    
    tmpl['DwcaListEmptyMessage'] = "No DarwinCore Archives created so far."
    tmpl['hideDwcaListStart'] = "<!--"
    tmpl['hideDwcaListEnd'] = "-->"
           
#
# print HTML !
#

printOverHTTP(tmpl)
